from __future__ import print_function

import os
import sys
 
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

os.environ['BURLAP_NO_LOAD'] = '1'

import burlap # pylint: disable=wrong-import-position

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print('warning: pypandoc module not found, could not convert Markdown to RST')
    read_md = lambda f: open(f, 'r').read()

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))

def read(filename):
    path = os.path.join(os.path.dirname(__file__), filename)
    with open(path, 'rb') as fin:
        text = fin.read().decode('utf-8')
    #data.decode("utf8", "ignore")
    return text
    
def get_reqs(fn):
    return [
        _.strip()
        for _ in open(os.path.join(CURRENT_DIR, fn)).readlines()
        if _.strip()
    ]

class Tox(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import tox
        tox.cmdline(self.test_args)
        sys.exit(0)
        
setup(
    name="burlap",
    version=burlap.__version__,
    packages=find_packages(exclude=['ez_setup', 'tests']),
    scripts=['bin/burlap-admin.py'],
    package_data={
        'burlap': [
            'templates/*.*',
            'fixtures/*.*',
        ],
    },
    author="Chris Spencer",
    author_email="chrisspen@gmail.com",
    description="Fabric commands for simplifying server deployments",
    long_description=read_md('README.md'),
    license="MIT",
    url="https://github.com/chrisspen/burlap",
    #https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        #'Development Status :: 3 - Alpha',
        'Development Status :: 4 - Beta',
        #'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
    ],
    zip_safe=False,
    include_package_data=True,
    install_requires=get_reqs('burlap/fixtures/requirements.txt'),
    setup_requires=[],
    tests_require=get_reqs('requirements-test.txt'),
    cmdclass={
        'test': Tox,
    },
)



import socket

from burlap.tests.functional_tests.base import TestCase
        
def is_valid_ipv4_address(address):
    try:
        _ = socket.inet_pton(socket.AF_INET, address)
    except AttributeError:  # no inet_pton here, sorry
        try:
            _ = socket.inet_aton(address)
        except socket.error:
            return False
        return address.count('.') == 3
    except socket.error:    # not a valid address
        return False
    return True


def is_valid_ipv6_address(address):
    try:
        _ = socket.inet_pton(socket.AF_INET6, address)
    except socket.error:    # not a valid address
        return False
    return True


def is_valid_address(address):
    return is_valid_ipv4_address(address) or is_valid_ipv6_address()

class NetworkTests(TestCase):
    
    def test_interfaces_ipv4_addresses(self):
    
        from burlap.network import interfaces
        from burlap.network import address
    
        for interface in interfaces():
            ipv4_address = address(interface)
            assert is_valid_ipv4_address(ipv4_address)
    
    
    def test_loopback_interface_exists(self):
        from burlap.network import interfaces
        assert 'lo' in interfaces()
    
    
    def test_loopback_interface_address(self):
        from burlap.network import address
        assert address('lo') == '127.0.0.1'
    
    
    def test_name_servers(self):
        from burlap.network import nameservers
        for address in nameservers():
            assert is_valid_address(address)



from __future__ import print_function

from burlap import Satchel
from burlap.constants import *
from burlap.decorators import task

class RsyncSatchel(Satchel):
    
    name = 'rsync'
    
    def set_defaults(self):
        self.env.clean = 1
        self.env.gzip = 1
        self.env.exclusions = [
            '*_local.py',
            '*.pyc',
            '*.pyo',
            '*.pyd',
            '*.svn',
            '*.tar.gz',
            #'static',
        ]
        self.src_dir = '.'
        #self.env.dir = '.burlap/rsync_cache'
        self.env.extra_dirs = []
        self.env.chown_user = 'www-data'
        self.env.chown_group = 'www-data'
        self.env.command = 'rsync --verbose --compress --recursive --delete ' \
            '--rsh "ssh -i {key_filename}" {exclusions_str} {rsync_src_dir} {user}@{host_string}:{rsync_dst_dir}'
    
    @task
    def deploy_code(self):
        """
        Generates a rsync of all deployable code.
        """
        
        assert self.genv.SITE, 'Site unspecified.'
        assert self.genv.ROLE, 'Role unspecified.'
        
        r = self.local_renderer
        
        if self.env.exclusions:
            r.env.exclusions_str = ' '.join(
                "--exclude='%s'" % _ for _ in self.env.exclusions)
        
        r.local(r.env.rsync_command)
        r.sudo('chown -R {rsync_chown_user}:{rsync_chown_group} {rsync_dst_dir}')

    @task
    def configure(self):
        pass

rsync_satchel = RsyncSatchel()



from __future__ import print_function

from burlap import ServiceSatchel
from burlap.constants import *
from burlap.decorators import task

class JSHintSatchel(ServiceSatchel):
    
    name = 'jshint'

    def set_defaults(self):
        pass

#     @property
#     def packager_system_packages(self):
#         return {
#             DEBIAN: ['npm'],
#             UBUNTU: ['npm'],
#         }
        
    @task
    def install_packages(self):
        packager = self.packager
        if packager == APT:
            self.sudo('apt-get update --fix-missing')
            # Necessary because Ubuntu 14 has malformed npm/nodejs packages.
            self.sudo('DEBIAN_FRONTEND=noninteractive apt-get -f -o Dpkg::Options::="--force-overwrite" install --yes npm')
        else:
            raise NotImplementedError('Unsupported packager: %s' % (packager,))
        
    @task(precursors=['packager', 'user'])
    def configure(self):
        r = self.local_renderer
        if r.env.enabled:
            self.install_packages()
            r.sudo('npm install -g jshint')
            # The Ubuntu 14 package is malformed and refers to "node" instead of "nodejs".
            r.sudo('[ ! -f /usr/bin/node ] && ln -s /usr/bin/nodejs /usr/bin/node || true')
        else:
            r.sudo('npm uninstall -g jshint')
    
jshint = JSHintSatchel()



from __future__ import print_function

#from fabric.contrib.files import exists
 
from burlap.common import set_verbose
from burlap.js import jshint
from burlap.tests.functional_tests.base import TestCase
from burlap.deploy import thumbprint, clear_fs_cache, delete_plan_data_dir
#from burlap.packager import packager

class JSTests(TestCase):
    
    def test_jshint(self):
        
        set_verbose(True)
        jshint.genv.ROLE = 'local'
        jshint.genv.services = ['jshint']
        jshint.clear_caches()
        
#         packager.update()
#         packager.upgrade(full=1)

        print('Installing jshint...')
        jshint.env.enabled = True
        jshint.clear_local_renderer()
        #jshint.install_packages() # fails on Ubuntu 14 under Travis-CI?
        #jshint.sudo('apt-get purge nodejs-legacy nodejs')
        #jshint.sudo('apt-get update --fix-missing; DEBIAN_FRONTEND=noninteractive apt-get -f -o Dpkg::Options::="--force-overwrite" install --yes npm')
        jshint.configure()
        clear_fs_cache()
        delete_plan_data_dir()
        print('-'*80)
        print('Thumbprinting...')
        thumbprint(components=jshint.name)
        clear_fs_cache()
        print('-'*80)
        
        # Confirm jshint was installed.
        #assert exists('/usr/local/bin/jshint')
        output = jshint.run('jshint --version')
        print('output:', output)
        assert 'jshint v' in output
        
        print('Disabling jshint...')
        jshint.env.enabled = False
        jshint.clear_local_renderer()
        jshint.configure()

        # Confirm jshint was uninstalled.
        #assert not exists('/usr/local/bin/jshint')




from collections import OrderedDict

import yaml

from fabric.api import env

class Shelf(object):
    """
    A helper class for serializing Python dictionaries to and from YAML,
    keeping the dictionary keys sorted to help readability.
    """
    
    def __init__(self, ascii_str=True, filename='roles/{role}/shelf.yaml'):
        
        # If true, automatically ensure all string values are plain ASCII.
        # This helps keep the YAML clean, otherwise verbose syntax would be
        # added for non-ASCII encodings, even if the string only contains
        # ASCII characters.
        self.ascii_str = ascii_str
        
        self._filename = filename
        
    @property
    def filename(self):
        return self._filename.format(role=env.ROLE.lower())

    @property
    def _dict(self):
        try:
            return OrderedDict(yaml.load(open(self.filename, 'rb')) or {})
        except IOError:
            return OrderedDict()

    def __getitem__(self, name):
        return self.get(name=name)

    def __setitem__(self, name, value):
        return self.set(name=name, value=value)

    def get(self, name, default=None):
        d = self._dict
        return d.get(name, default)
    
    def setdefault(self, name, default):
        d = self._dict
        d.setdefault(name, default)
        yaml.dump(d, open(self.filename, 'wb'))
    
    def set(self, name, value):
        d = self._dict
        if self.ascii_str and isinstance(value, basestring):
            value = str(value)
        d[name] = value
        yaml.dump(d, open(self.filename, 'wb'))



import textwrap

from mock import patch

from burlap.tests.base import TestCase

class TestParseVagrantMachineReadableBoxList(TestCase):

    def test_machine_readable_box_list(self):
        with patch('burlap.vagrant.vagrant.local') as mock_local:
            mock_local.return_value = textwrap.dedent(r"""
                1391708688,,box-name,precise64
                1391708688,,box-provider,virtualbox
                """)
            from burlap.vagrant import vagrant
            res = vagrant._box_list_machine_readable()
            self.assertEqual(res, [
#                 ('lucid32', 'virtualbox'),
                ('precise64', 'virtualbox'),
#                 ('precise64', 'vmware_fusion'),
            ])


class TestParseVagrantBoxListWithProvider(TestCase):

    def test_parse_box_list(self):
        with patch('burlap.vagrant.vagrant.local') as mock_local:
            mock_local.return_value = textwrap.dedent("""\
                precise64                 (virtualbox)
                """)
            from burlap.vagrant import vagrant
            res = vagrant._box_list_human_readable()
            self.assertEqual(res, [
#                 ('lucid32', 'virtualbox'),
                ('precise64', 'virtualbox'),
#                 ('precise64', 'vmware_fusion'),
            ])


class TestParseVagrantBoxListWithoutProvider(TestCase):

    def test_parse_box_list(self):
        with patch('burlap.vagrant.vagrant.local') as mock_local:
            mock_local.return_value = textwrap.dedent("""\
                precise64
                """)
            from burlap.vagrant import vagrant
            res = vagrant._box_list_human_readable()
            self.assertEqual(res, [
#                 ('lucid32', 'virtualbox'),
                ('precise64', 'virtualbox'),
            ])


class TestVagrantBaseBoxes(TestCase):
 
    def test_vagrant_base_boxes(self):
        with patch('burlap.vagrant.vagrant._box_list') as mock_list:
            mock_list.return_value = [
                ('lucid32', 'virtualbox'),
                ('precise64', 'virtualbox'),
            ]
            from burlap.vagrant import vagrant
            self.assertEqual(vagrant.base_boxes(), ['lucid32', 'precise64'])



from __future__ import print_function

import os
import sys
import unittest
# from pprint import pprint

from burlap.common import set_state, get_state, clear_state, init_env, default_env, env, all_satchels, get_dryrun, set_dryrun, get_verbose, set_verbose
from burlap.deploy import init_env as deploy_init_env, delete_plan_data_dir, clear_fs_cache

def clear_runs_once(func):
    if hasattr(func, 'return_value'):
        print('clearing runs_once on %s' % func)
        print('return_value:', func.return_value)
        print('return_value:', func.wrapped.return_value)
        print('__dict__:', func.__dict__)
        # Fabric wraps function using a class that passes through get/hasattr
        # so we have to try deleting the attribute on a few levels.
        try:
            delattr(func, 'return_value')
        except AttributeError:
            pass
        try:
            delattr(func.wrapped, 'return_value')
        except AttributeError:
            pass
        assert not hasattr(func, 'return_value'), 'Unable to clear runs_once on %s' % func

class TestCase(unittest.TestCase):    
       
    test_name_fout = sys.stdout
 
    test_name_format = '\n{bar}\nRunning test: {name}\n{bar}\n'
    
    # These keys will not be cleared between tests.
    # This is useful for keeping the Vagrant login, which is only set once before all the tests are run.
    keep_env_keys = []
    
    def get_keep_env_keys(self):
        return list(self.keep_env_keys)
    
    def clear_env(self):
        keep_env_keys = set(self.get_keep_env_keys())
        for k, v in env.items():
            if k in keep_env_keys:
                continue
            del env[k]
    
    def update_env(self, d):
        keep_env_keys = set(self.get_keep_env_keys())
        for k, v in default_env.items():
            if k in keep_env_keys:
                continue
            env[k] = v
    
    def setUp(self):
        # Always print the current test name before the test.
        rows, columns = map(int, os.popen('stty size', 'r').read().split())
        kwargs = dict(
            bar='#'*columns,
            name=self._testMethodName,
        )
        print(self.test_name_format.format(**kwargs), file=self.test_name_fout)
        
        # Save fabric state.
        self._env = env.copy()
#         print('before env clear:')
#         pprint(env, indent=4)

        # Reset fabric state.
        #self.clear_env()
        #self.update_env(default_env)
        init_env()
        deploy_init_env()
        
        # Save burlap state.
        self._burlap_state = get_state()
        
        self._dryrun = get_dryrun()
        self._verbose = get_verbose()
        
        # Clear runs_once on legacy runs_once methods.
        from burlap import deploy, manifest
        modules = [deploy, manifest]
        for module in modules:
            for name in dir(module):
                func = getattr(module, name)
                if not callable(func):
                    continue
                clear_runs_once(func)
                
        # Clear runs_once on our custom runs_once methods.
        from burlap.common import runs_once_methods
        for meth in runs_once_methods:
            clear_runs_once(func)
        
        # Ensure all satchels re-push all their local variables back into the global env.
        for satchel in all_satchels.values():
            satchel.register()
            satchel.clear_caches()

        # Since these tests are automated, if we ever get a prompt, we should immediately fail,
        # because no action should ever be user-interactive.
        env.abort_on_prompts = True
        env.always_use_pty = False

        delete_plan_data_dir()

        clear_fs_cache()
        
        super(TestCase, self).setUp()

    def tearDown(self):
        
        set_dryrun(self._dryrun)
        set_verbose(self._verbose)
        
        # Restore fabric state.
        self.clear_env()
        self.update_env(self._env)
        
        # Restore burlap state.
        clear_state()
        set_state(self._burlap_state)
        
        super(TestCase, self).tearDown()
